// Generated by CoffeeScript 2.3.2
// # Row operations: CRUD operation on rows and columns

// Row objects provide access and manipulation on colunns and rows. Single and
// multiple operations are available and are documented below.

// ## Dependencies
var Row, Table, utils;

utils = require('./utils');

Table = require('./table');

// ## Constructor
Row = function(client, table, key) {
  this.client = client;
  this.table = (typeof table === "string" ? table : table.name);
  this.key = key;
  return this;
};

// ## `row.get`

// Retrieve values from HBase or retrieve values from multiple rows.
Row.prototype.get = function(column, callback) {
  var args, columns, end, isGlob, key, options, params, start, url;
  args = Array.prototype.slice.call(arguments);
  key = "/" + this.table + "/" + this.key;
  isGlob = this.key.substr(-1, 1) === "*";
  options = {};
  columns = null;
  start = null;
  end = null;
  params = {};
  if (typeof args[0] === "string" || (typeof args[0] === "object" && args[0] instanceof Array)) {
    columns = args.shift();
  }
  if (typeof args[0] === "object") {
    options = args.shift();
  }
  if (options.start) {
    start = options.start;
  }
  if (options.end) {
    end = options.end;
  }
  if (options.v) {
    params.v = options.v;
  }
  url = utils.url.encode({
    table: this.table,
    key: this.key,
    columns: columns,
    start: start,
    end: end,
    params: params
  });
  return this.client.connection.get(url, (error, data) => {
    var cells;
    if (error) {
      return args[0].apply(this, [error, null]);
    }
    cells = [];
    data.Row.forEach((row) => {
      key = utils.base64.decode(row.key, this.client.options.encoding);
      return row.Cell.forEach((cell) => {
        data = {};
        if (isGlob) {
          data.key = key;
        }
        data.column = utils.base64.decode(cell.column, this.client.options.encoding);
        data.timestamp = cell.timestamp;
        data.$ = utils.base64.decode(cell.$, this.client.options.encoding);
        return cells.push(data);
      });
    });
    return args[0].apply(this, [null, cells]);
  });
};

// ## `row.put`

// Insert and update a column value, insert and update multiple column values or insert and update multiple rows.
Row.prototype.put = function(columns, values, callback) {
  var args, body, bodyCell, bodyRow, cell, cells, cellsKeys, data, k, k1, timestamps, url;
  args = Array.prototype.slice.call(arguments);
  url = void 0;
  body = void 0;
  bodyRow = void 0;
  // First argument are columns and second argument are values
  if (args.length > 2) {
    columns = args.shift();
    values = args.shift();
    timestamps = void 0;
    if (typeof args[0] !== "function") {
      timestamps = args.shift();
    }
    callback = args.shift();
    if (typeof columns === "string") {
      columns = [columns];
      values = [values];
    } else {
      if (columns.length !== values.length) {
        throw new Error("Columns count must match values count");
      }
    }
    body = {
      Row: []
    };
    bodyRow = {
      key: utils.base64.encode(this.key, this.client.options.encoding),
      Cell: []
    };
    columns.forEach((column, i) => {
      var bodyCell;
      bodyCell = {};
      if (timestamps) {
        bodyCell.timestamp = timestamps[i];
      }
      bodyCell.column = utils.base64.encode(column, this.client.options.encoding);
      bodyCell.$ = utils.base64.encode(values[i], this.client.options.encoding);
      return bodyRow.Cell.push(bodyCell);
    });
    body.Row.push(bodyRow);
    url = utils.url.encode({
      table: this.table,
      key: this.key || "___false-row-key___",
      columns: columns
    });
  } else {
    // First argument is a full object with columns and values
    data = args.shift();
    callback = args.shift();
    body = {
      Row: []
    };
    cellsKeys = {};
    data.forEach((d) => {
      var key;
      key = d.key || this.key;
      if (!(key in cellsKeys)) {
        cellsKeys[key] = [];
      }
      return cellsKeys[key].push(d);
    });
    for (k in cellsKeys) {
      cells = cellsKeys[k];
      bodyRow = {
        key: utils.base64.encode(k, this.client.options.encoding),
        Cell: []
      };
      for (k1 in cells) {
        cell = cells[k1];
        bodyCell = {};
        if (cell.timestamp) {
          bodyCell.timestamp = "" + cell.timestamp;
        }
        bodyCell.column = utils.base64.encode(cell.column, this.client.options.encoding);
        bodyCell.$ = utils.base64.encode(cell.$, this.client.options.encoding);
        bodyRow.Cell.push(bodyCell);
      }
      body.Row.push(bodyRow);
    }
    url = utils.url.encode({
      table: this.table,
      key: this.key || "___false-row-key___",
      columns: ['test:']
    });
  }
  return this.client.connection.put(url, body, (error, data) => {
    if (!callback) {
      return;
    }
    return callback.apply(this, [error, (error ? null : true)]);
  });
};

// ## `row.exists`

// Test if a row or a column exists
Row.prototype.exists = function(column, callback) {
  var args, url;
  args = Array.prototype.slice.call(arguments);
  column = (typeof args[0] === "string" ? args.shift() : null);
  url = utils.url.encode({
    table: this.table,
    key: this.key,
    columns: column
  });
  return this.client.connection.get(url, (error, exists) => {
    // note:
    // if row does not exists: 404
    // if row exists and column family does not exists: 503
    if (error && (error.code === 404 || error.code === 503)) {
      error = null;
      exists = false;
    }
    return args[0].apply(this, [error, (error ? null : (exists === false ? false : true))]);
  });
};

// ## `row.delete`

// Delete a row or column or delete multiple columns.
Row.prototype.delete = function() {
  var args, columns, url;
  args = Array.prototype.slice.call(arguments);
  columns = void 0;
  if (typeof args[0] === "string" || (typeof args[0] === "object" && args[0] instanceof Array)) {
    columns = args.shift();
  }
  url = utils.url.encode({
    table: this.table,
    key: this.key,
    columns: columns
  });
  return this.client.connection.delete(url, ((error, success) => {
    return args[0].apply(this, [error, (error ? null : true)]);
  }), true);
};

module.exports = Row;
